local *

{
  :AcceptResurrect
  :C_DeathInfo
  :C_PetJournal
  :C_SummonInfo
  :CanMerchantRepair
  :ConfirmSummon
  :Dismount
  :ERR_ATTACK_MOUNTED
  :ERR_MOUNT_ALREADYMOUNTED
  :ERR_NOT_WHILE_MOUNTED
  :ERR_TAXIPLAYERALREADYMOUNTED
  :GetCoinTextureString
  :GetContainerItemLink
  :GetContainerNumSlots
  :GetCorpseRecoveryDelay
  :GetItemInfo
  :GetMoney
  :GetRepairAllCost
  :IsInInstance
  :IsMounted
  :RepairAllItems
  :RepopMe
  :SPELL_FAILED_NOT_MOUNTED
  :StaticPopup1
  :StaticPopup1Button1
  :UnitAffectingCombat
  :UseContainerItem
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :ToggleOption
} = idTweaksOptions

class Automation extends idTweaksModule
  mount_error_messages:
    [ERR_ATTACK_MOUNTED]: true
    [ERR_MOUNT_ALREADYMOUNTED]: true
    [ERR_NOT_WHILE_MOUNTED]: true
    [ERR_TAXIPLAYERALREADYMOUNTED]: true
    [SPELL_FAILED_NOT_MOUNTED]: true
  
  new: (addon) =>
    -- print "Automation#new"

    super addon, 'Automation', 'AceConsole-3.0', 'AceEvent-3.0'

  set_options: =>
    -- print "Automation#set_options"

    super!

    @options\add_option Option @, 'accepting', 'Accepting', 'Automatically accept certain offers.', {},
      accept_hearthstone_bind: ToggleOption @, 'accept_hearthstone_bind', 'Accept Hearthstone Bind', 'Automatically accepts HearthStone binds'
      accept_pvp_release: ToggleOption @, 'accept_pvp_release', 'Accept PVP Release', 'Automatically accepts PVP Release'
      accept_resurrection: ToggleOption @, 'accept_resurrection', 'Accept Resurrection', 'Automatically accepts Resurrections'
      accept_summon: ToggleOption @, 'accept_summon', 'Accept Summon', 'Automatically accepts Summons (disabled by default for raiders)'

    @options\add_option Option @, 'vendors', 'Vendors', 'Automate Vendor Interactions.', {},
      repair_gear: ToggleOption @, 'repair_gear', 'Repair Gear', 'Automatically repairs your gear'
      sell_grey_items: ToggleOption @, 'sell_grey_items', 'Sell Grey Items', 'Automatically sells grey items'

    if idTweaks\is_retail!
      @options\add_option Option @, 'minipet', 'Minipet', 'Minipet', {},
        summon: ToggleOption @, 'summon', 'Summon', 'Summon Random Favorite Battlepet'

    if idTweaks\is_bc!
      @options\add_option Option @, 'movement', 'Movement', 'Movement', {},
        dismount: ToggleOption @, 'dismount', 'Dismount', 'Dismount when needed'

  enable: =>
    -- print "Automation#enable"

    @ace_module\RegisterEvent 'CONFIRM_BINDER', (...) -> @CONFIRM_BINDER ...
    @ace_module\RegisterEvent 'CONFIRM_SUMMON', (...) -> @CONFIRM_SUMMON ...
    @ace_module\RegisterEvent 'MERCHANT_SHOW', (...) -> @MERCHANT_SHOW ...
    @ace_module\RegisterEvent 'PLAYER_DEAD', (...) -> @PLAYER_DEAD ...
    @ace_module\RegisterEvent 'RESURRECT_REQUEST', (...) -> @RESURRECT_REQUEST ...
    
    if idTweaks\is_retail!
      @ace_module\RegisterEvent 'PLAYER_ALIVE', (...) -> @PLAYER_ALIVE ...
      @ace_module\RegisterEvent 'PLAYER_CONTROL_GAINED', (...) -> @PLAYER_CONTROL_GAINED ...
      @ace_module\RegisterEvent 'PLAYER_ENTERING_WORLD', (...) -> @PLAYER_ENTERING_WORLD ...
      @ace_module\RegisterEvent 'PLAYER_MOUNT_DISPLAY_CHANGED', (...) -> @PLAYER_MOUNT_DISPLAY_CHANGED ...
      @ace_module\RegisterEvent 'PLAYER_UNGHOST', (...) -> @PLAYER_UNGHOST ...

    if idTweaks\is_bc!
      @ace_module\RegisterEvent 'UI_ERROR_MESSAGE', (...) -> @UI_ERROR_MESSAGE ...

  disable: =>
    -- print "Automation#disable"

    @ace_module\UnregisterEvent 'CONFIRM_BINDER'
    @ace_module\UnregisterEvent 'CONFIRM_SUMMON'
    @ace_module\UnregisterEvent 'MERCHANT_SHOW'
    @ace_module\UnregisterEvent 'PLAYER_DEAD'
    @ace_module\UnregisterEvent 'RESURRECT_REQUEST'
    
    if idTweaks\is_retail!
      @ace_module\UnregisterEvent 'PLAYER_ALIVE'
      @ace_module\UnregisterEvent 'PLAYER_CONTROL_GAINED'
      @ace_module\UnregisterEvent 'PLAYER_ENTERING_WORLD'
      @ace_module\UnregisterEvent 'PLAYER_MOUNT_DISPLAY_CHANGED'
      @ace_module\UnregisterEvent 'PLAYER_UNGHOST'

    if idTweaks\is_bc!
      @ace_module\UnregisterEvent 'UI_ERROR_MESSAGE'

  summon_battlepet: =>
    -- print "Automation#summon_battlepet"

  sell_grey_items: =>
    -- print "Automation#sell_grey_items"

    for bag = 0, 4
      for slot = 0, GetContainerNumSlots bag
        link = GetContainerItemLink bag, slot
        if link and (select 3, GetItemInfo link) == 0
          UseContainerItem bag, slot

  repair_gear: =>
    -- print "Automation#repair_gear"

    cost = GetRepairAllCost!
    funds = GetMoney!

    if cost == 0
      @ace_module\Print 'Repair: Nothing to repair'

    if funds < cost
      @ace_module\Print 'Repair: Insufficient Funds'
      return

    if cost > 0
      @ace_module\Print "Repair: #{GetCoinTextureString cost}"
      RepairAllItems!

  CONFIRM_BINDER: =>
    -- print "Automation#CONFIRM_BINDER"

    if @db.profile.accepting.accept_hearthstone_bind == true
      StaticPopup1Button1\Click!

  CONFIRM_SUMMON: =>
    -- print "Automation#CONFIRM_SUMMON"

    return unless @db.profile.accepting.accept_summon
    return if UnitAffectingCombat 'player'

    StaticPopup1\Hide!
    C_SummonInfo.ConfirmSummon!

  MERCHANT_SHOW: =>
    -- print "Automation#MERCHANT_SHOW"

    @sell_grey_items! if @db.profile.vendors.sell_grey_items == true
    @repair_gear! if CanMerchantRepair! and @db.profile.vendors.repair_gear == true

  PLAYER_DEAD: =>
    -- print "Automation#PLAYER_DEAD"

    return unless @db.profile.accepting.accept_pvp_release
    return if UnitAffectingCombat 'player'
    return if C_DeathInfo.GetSelfResurrectOptions! and #C_DeathInfo.GetSelfResurrectOptions! > 0

    in_instance, instance_type = IsInInstance!

    RepopMe! if in_instance and instance_type == 'pvp'

  RESURRECT_REQUEST: =>
    -- print "Automation#RESURRECT_REQUEST"

    return unless @db.profile.accepting.accept_resurrection
    return if GetCorpseRecoveryDelay! ~= 0

    StaticPopup1\Hide!
    AcceptResurrect!

  -- PLAYER_CONTROL_GAINED: =>
  --   -- print "Automation#PLAYER_CONTROL_GAINED"
  --   return if not @db.profile.minipet.summon
  --   return if UnitAffectingCombat 'player'
  --   @summon_battlepet!

    return unless idTweaks\is_retail!
    return unless @db.profile.minipet.summon
    return if UnitAffectingCombat 'player'

  -- PLAYER_MOUNT_DISPLAY_CHANGED: =>
  --   -- print "Automation#PLAYER_MOUNT_DISPLAY_CHANGED"
  --   return if not @db.profile.minipet.summon
  --   return if UnitAffectingCombat 'player'
  --   @summon_battlepet!

  -- PLAYER_ALIVE: =>
  --   -- print "Automation#PLAYER_ALIVE"
  --   return if not @db.profile.minipet.summon
  --   return if UnitAffectingCombat 'player'
  --   @summon_battlepet!

    return unless idTweaks\is_retail!
    return unless @db.profile.minipet.summon
    return if UnitAffectingCombat 'player'

  mountErrorMessages:
    [ERR_ATTACK_MOUNTED]: true
    [ERR_MOUNT_ALREADYMOUNTED]: true
    [ERR_NOT_WHILE_MOUNTED]: true
    [ERR_TAXIPLAYERALREADYMOUNTED]: true
    [SPELL_FAILED_NOT_MOUNTED]: true

  PLAYER_ALIVE: =>
    -- print "Automation#PLAYER_ALIVE"

    return unless idTweaks\is_retail!
    return unless @db.profile.minipet.summon
    return if UnitAffectingCombat 'player'

    @summon_battlepet!

  PLAYER_UNGHOST: =>
    -- print "Automation#PLAYER_UNGHOST"

    return unless idTweaks\is_retail!
    return unless @db.profile.minipet.summon
    return if UnitAffectingCombat 'player'

    @summon_battlepet!
    
  UI_ERROR_MESSAGE: (_, _, errorMessage) =>
    -- print "Automation#UI_ERROR_MESSAGE"

    return unless idTweaks\is_bc!
    return unless @db.profile.movement.dismount
    return unless IsMounted!

    if @mount_error_messages[errorMessage]
      Dismount!

idTweaks\add_module Automation idTweaks
